@@ -1,7 +1,7 @@
 //! Basic font attributes: stretch, weight and style.
 
 use super::internal::{head::Os2, RawFont};
-use super::{tag_from_bytes, FontRef, Tag, Setting};
+use super::{tag_from_bytes, FontRef, Setting, Tag};
 
 use core::fmt;
 use core::hash::{Hash, Hasher};
@@ -13,15 +13,15 @@ const SLNT: Tag = tag_from_bytes(b"slnt");
 const ITAL: Tag = tag_from_bytes(b"ital");
 
 /// Primary attributes for font classification: stretch, weight and style.
-/// 
+///
 /// This struct is created by the [`attributes`](FontRef::attributes) method on [`FontRef`].
 #[derive(Copy, Clone)]
-pub struct Attributes(pub(crate) u32);
+pub struct Attributes(pub u32);
 
 impl Attributes {
     /// Creates new font attributes from the specified stretch, weight and
     /// style.
-    pub fn new(stretch: Stretch, weight: Weight, style: Style) -> Self {
+    pub const fn new(stretch: Stretch, weight: Weight, style: Style) -> Self {
         let stretch = stretch.0 as u32 & 0x1FF;
         let weight = weight.0 as u32 & 0x3FF;
         let style = style.pack();
@@ -95,21 +95,21 @@ impl Attributes {
         var_bits & 1 != 0
     }
 
-    /// Returns true if the font has a variation for the weight attribute.    
+    /// Returns true if the font has a variation for the weight attribute.
     pub fn has_weight_variation(&self) -> bool {
         let var_bits = self.0 >> 28;
         var_bits & 2 != 0
     }
 
     /// Returns true if the font has a variation for the oblique style
-    /// attribute.    
+    /// attribute.
     pub fn has_oblique_variation(&self) -> bool {
         let var_bits = self.0 >> 28;
         var_bits & 4 != 0
     }
 
     /// Returns true if the font has a variation for the italic style
-    /// attribute.    
+    /// attribute.
     pub fn has_italic_variation(&self) -> bool {
         let var_bits = self.0 >> 28;
         var_bits & 8 != 0
@@ -127,14 +127,20 @@ impl Attributes {
             let stretch = self.stretch();
             let req_stretch = requested.stretch();
             if stretch != requested.stretch() {
-                synth.vars[len] = Setting { tag: WDTH, value: req_stretch.to_percentage() };
+                synth.vars[len] = Setting {
+                    tag: WDTH,
+                    value: req_stretch.to_percentage(),
+                };
                 len += 1;
             }
         }
         let (weight, req_weight) = (self.weight(), requested.weight());
         if weight != req_weight {
             if self.has_weight_variation() {
-                synth.vars[len] = Setting { tag: WGHT, value: req_weight.0 as f32 };
+                synth.vars[len] = Setting {
+                    tag: WGHT,
+                    value: req_weight.0 as f32,
+                };
                 len += 1;
             } else if req_weight > weight {
                 synth.embolden = true;
@@ -145,36 +151,42 @@ impl Attributes {
             match req_style {
                 Style::Normal => {}
                 Style::Italic => {
-                    match style {
-                        Style::Normal => {
-                            if self.has_italic_variation() {
-                                synth.vars[len] = Setting { tag: ITAL, value: 1. };
-                                len += 1;
-                            } else if self.has_oblique_variation() {
-                                synth.vars[len] = Setting { tag: SLNT, value: 14. };
-                                len += 1;
-                            } else {
-                                synth.skew = 14;
-                            }
+                    if style == Style::Normal {
+                        if self.has_italic_variation() {
+                            synth.vars[len] = Setting {
+                                tag: ITAL,
+                                value: 1.,
+                            };
+                            len += 1;
+                        } else if self.has_oblique_variation() {
+                            synth.vars[len] = Setting {
+                                tag: SLNT,
+                                value: 14.,
+                            };
+                            len += 1;
+                        } else {
+                            synth.skew = 14;
                         }
-                        _ => {}
                     }
                 }
                 Style::Oblique(angle) => {
-                    match style {
-                        Style::Normal => {
-                            let degrees = angle.to_degrees();
-                            if self.has_oblique_variation() {
-                                synth.vars[len] = Setting { tag: SLNT, value: degrees };
-                                len += 1;
-                            } else if self.has_italic_variation() && degrees > 0. {
-                                synth.vars[len] = Setting { tag: ITAL, value: 1. };
-                                len += 1;
-                            } else {
-                                synth.skew = degrees as i8;
-                            }
+                    if style == Style::Normal {
+                        let degrees = angle.to_degrees();
+                        if self.has_oblique_variation() {
+                            synth.vars[len] = Setting {
+                                tag: SLNT,
+                                value: degrees,
+                            };
+                            len += 1;
+                        } else if self.has_italic_variation() && degrees > 0. {
+                            synth.vars[len] = Setting {
+                                tag: ITAL,
+                                value: 1.,
+                            };
+                            len += 1;
+                        } else {
+                            synth.skew = degrees as i8;
                         }
-                        _ => {}
                     }
                 }
             }
@@ -368,7 +380,7 @@ impl Style {
     }
 
     /// Returns the angle of the style in degrees.
-    pub fn to_degrees(&self) -> f32 {
+    pub fn to_degrees(self) -> f32 {
         match self {
             Self::Italic => 14.,
             Self::Oblique(angle) => angle.to_degrees(),
@@ -386,7 +398,7 @@ impl Style {
         }
     }
 
-    fn pack(&self) -> u32 {
+    const fn pack(&self) -> u32 {
         match self {
             Self::Normal => 0b10,
             Self::Italic => 0b110,
@@ -533,7 +545,7 @@ impl Stretch {
             "extra-expanded" => Self::EXTRA_EXPANDED,
             "ultra-expanded" => Self::ULTRA_EXPANDED,
             _ => {
-                if s.ends_with("%") {
+                if s.ends_with('%') {
                     let p = s.get(..s.len() - 1)?.parse::<f32>().ok()?;
                     return Some(Self::from_percentage(p));
                 }
@@ -598,8 +610,8 @@ impl Default for Stretch {
     }
 }
 
-/// Synthesis suggestions for mistmatched font attributes.
-/// 
+/// Synthesis suggestions for mismatched font attributes.
+///
 /// This is generated by the [`synthesize`](Attributes::synthesize) method on
 /// [`Attributes`].
 #[derive(Copy, Clone, Default)]
