@@ -8,6 +8,9 @@ pub const KERX: RawTag = raw_tag(b"kerx");
 pub const ANKR: RawTag = raw_tag(b"ankr");
 pub const KERN: RawTag = raw_tag(b"kern");
 
+/// Maximum number of times we allow consecutive DONT_ADVANCE states.
+const MAX_CYCLES: u16 = 16;
+
 /// Gets a value from a lookup table.
 ///
 /// <https://developer.apple.com/fonts/TrueType-Reference-Manual/RM06/Chap6Tables.html>
@@ -67,15 +70,14 @@ pub fn lookup<T: FromBeData>(data: &Bytes, offset: usize, id: u16) -> Option<T>
             let mut l = 0;
             let mut h = nrecs;
             while l < h {
+                use core::cmp::Ordering::*;
                 let i = (l + h) / 2;
                 let rec = base + i * reclen;
                 let glyph = data.read::<u16>(rec)?;
-                if id > glyph {
-                    l = i + 1;
-                } else if id < glyph {
-                    h = i;
-                } else {
-                    return data.read::<T>(rec + 2);
+                match id.cmp(&glyph) {
+                    Greater => l = i + 1,
+                    Less => h = i,
+                    Equal => return data.read::<T>(rec + 2),
                 }
             }
         }
@@ -162,7 +164,7 @@ pub struct StateTable<'a> {
     classes: &'a [u8],
 }
 
-impl <'a> StateTable<'a> {
+impl<'a> StateTable<'a> {
     /// Creates a new extended state table for the specified data.
     pub fn new(data: &Bytes<'a>) -> Option<Self> {
         let class_count = data.read_u16(0)?;
@@ -194,19 +196,21 @@ impl <'a> StateTable<'a> {
         } else {
             1
         }
-    }    
+    }
 
     /// Returns the entry for the specified state and class.
     pub fn entry<T: FromBeData>(&self, state: u16, class: u16) -> Option<Entry<T>> {
         let index = state as usize * self.class_count as usize + class as usize;
         let entry_index = *self.states.get(index)? as usize;
         let entry_offset = entry_index * (4 + T::SIZE);
-        let mut entry = self.data
+        let mut entry = self
+            .data
             .read::<Entry<T>>(self.entry_table as usize + entry_offset)?;
-        let new_state = (entry.new_state as u32).checked_sub(self.state_array)? / self.class_count as u32;
+        let new_state =
+            (entry.new_state as u32).checked_sub(self.state_array)? / self.class_count as u32;
         entry.new_state = new_state as u16;
         Some(entry)
-    }    
+    }
 }
 
 /// Entry in a state table.
@@ -239,7 +243,7 @@ pub mod morx {
     /// Returns an iterator over the chains in a `morx` table at the
     /// specified offset.
     pub fn chains<'a>(data: &'a [u8], offset: u32) -> Chains<'a> {
-        let data = Bytes::with_offset(data, offset as usize).unwrap_or(Bytes::new(&[]));
+        let data = Bytes::with_offset(data, offset as usize).unwrap_or_else(|| Bytes::new(&[]));
         let len = data.read_u32(4).unwrap_or(0);
         Chains {
             data,
@@ -454,7 +458,7 @@ pub mod morx {
                 0 => SubtableKind::Rearrangement(Rearrangement::new(data)?),
                 1 => SubtableKind::Contextual(Contextual::new(data)?),
                 2 => SubtableKind::Ligature(Ligature::new(data)?),
-                4 => SubtableKind::NonContextual(NonContextual::new(data)?),
+                4 => SubtableKind::NonContextual(NonContextual::new(data)),
                 5 => SubtableKind::Insertion(Insertion::new(data)?),
                 _ => return None,
             })
@@ -491,12 +495,17 @@ pub mod morx {
             state: &mut RearrangementState,
             index: usize,
             glyph_id: u16,
+            end_of_text: bool,
             mut f: impl FnMut(&Rearrange) -> Option<()>,
         ) -> Option<usize> {
             const MARK_FIRST: u16 = 0x8000;
             const DONT_ADVANCE: u16 = 0x4000;
             const MARK_LAST: u16 = 0x2000;
-            let class = self.state_table.class(glyph_id);
+            let class = if end_of_text {
+                0
+            } else {
+                self.state_table.class(glyph_id)
+            };
             let entry = self.state_table.entry::<()>(state.state, class)?;
             state.state = entry.new_state;
             if entry.flags & MARK_FIRST != 0 {
@@ -508,7 +517,7 @@ pub mod morx {
             let verb = entry.flags & 0xF;
             let start = state.first;
             let end = state.last;
-            if verb != 0 && start < end {
+            if verb != 0 && start <= end {
                 let m = REARRANGEMENT_MAP[verb as usize & 0xF];
                 let l = 2.min(m >> 4) as usize;
                 let r = 2.min(m & 0x0F) as usize;
@@ -524,7 +533,15 @@ pub mod morx {
                 };
                 f(&rearrange)?;
             }
-            let advance = entry.flags & DONT_ADVANCE == 0;
+            let mut advance = entry.flags & DONT_ADVANCE == 0;
+            if advance {
+                state.cycles = 0;
+            } else if state.cycles > MAX_CYCLES {
+                state.cycles = 0;
+                advance = true;
+            } else {
+                state.cycles += 1;
+            }
             Some(advance as usize)
         }
     }
@@ -554,6 +571,7 @@ pub mod morx {
         state: u16,
         first: usize,
         last: usize,
+        cycles: u16,
     }
 
     impl RearrangementState {
@@ -584,28 +602,14 @@ pub mod morx {
             let start = self.start;
             let end = (self.end + 1).min(buffer.len());
             let mut tmp = [T::default(); 4];
-            if end - start >= usize::from(l + r) {
-                for i in 0..l {
-                    tmp[i] = buffer[start + i];
-                }
-                for i in 0..r {
-                    tmp[i + 2] = buffer[end - r + i];
-                }
-                if l > r {
-                    for i in 0..(end - start - l - r) {
-                        buffer[start + r + i] = buffer[start + l + i];
-                    }
-                } else if l < r {
-                    for i in (0..(end - start - l - r)).rev() {
-                        buffer[start + r + i] = buffer[start + l + i];
-                    }
-                }
-                for i in 0..r {
-                    buffer[start + i] = tmp[2 + i];
-                }
-                for i in 0..l {
-                    buffer[end - l + i] = tmp[i];
+            if end - start >= l + r {
+                tmp[..l].copy_from_slice(&buffer[start..(start + l)]);
+                tmp[2..(2 + r)].copy_from_slice(&buffer[(end - r)..end]);
+                if l != r {
+                    buffer.copy_within((start + l)..(end - r), start + r);
                 }
+                buffer[start..(r + start)].copy_from_slice(&tmp[2..(r + 2)]);
+                buffer[(end - l)..end].copy_from_slice(&tmp[..l]);
                 if reverse_l {
                     buffer.swap(end - 1, end - 2);
                 }
@@ -640,13 +644,19 @@ pub mod morx {
             state: &mut ContextualState,
             index: usize,
             glyph_id: u16,
+            end_of_text: bool,
             mut f: impl FnMut(usize, u16) -> Option<()>,
         ) -> Option<()> {
             const SET_MARK: u16 = 0x8000;
             const DONT_ADVANCE: u16 = 0x4000;
             let mut last_glyph_id = glyph_id;
             let mut current_glyph_id = glyph_id;
-            let mut class = self.state_table.class(glyph_id);
+            let mut class = if end_of_text {
+                0
+            } else {
+                self.state_table.class(glyph_id)
+            };
+            let mut cycles = 0;
             loop {
                 let entry = self
                     .state_table
@@ -668,9 +678,10 @@ pub mod morx {
                 if entry.flags & SET_MARK != 0 {
                     state.mark = Some((index, glyph_id));
                 }
-                if entry.flags & DONT_ADVANCE == 0 {
+                if entry.flags & DONT_ADVANCE == 0 || cycles > MAX_CYCLES {
                     break;
                 }
+                cycles += 1;
                 if current_glyph_id != last_glyph_id {
                     class = self.state_table.class(current_glyph_id);
                 }
@@ -751,6 +762,7 @@ pub mod morx {
             state: &mut LigatureState,
             index: usize,
             glyph_id: u16,
+            end_of_text: bool,
             mut f: impl FnMut(usize, u16, &[usize]) -> Option<()>,
         ) -> Option<()> {
             const SET_COMPONENT: u16 = 0x8000;
@@ -758,7 +770,12 @@ pub mod morx {
             const PERFORM_ACTION: u16 = 0x2000;
             const LAST: u32 = 0x80000000;
             const STORE: u32 = 0x40000000;
-            let class = self.state_table.class(glyph_id);
+            let class = if end_of_text {
+                0
+            } else {
+                self.state_table.class(glyph_id)
+            };
+            let mut cycles = 0;
             loop {
                 let entry = self.state_table.entry::<u16>(state.state, class)?;
                 state.state = entry.new_state;
@@ -787,15 +804,17 @@ pub mod morx {
                             let ligature = self.ligature(ligature_index)?;
                             f(glyph_index, ligature, state.indices.get(pos + 1..end_pos)?)?;
                         }
-                        if action & LAST != 0 {
+                        if action & LAST != 0 || cycles > (MAX_CYCLES * 2) {
                             break;
                         }
+                        cycles += 1;
                     }
                     state.pos = pos;
                 }
-                if entry.flags & DONT_ADVANCE == 0 {
+                if entry.flags & DONT_ADVANCE == 0 || cycles > MAX_CYCLES {
                     break;
                 }
+                cycles += 1;
             }
             Some(())
         }
@@ -840,7 +859,7 @@ pub mod morx {
             *self.indices.get_mut(self.pos)? = index;
             *self.glyphs.get_mut(self.pos)? = glyph_id;
             self.pos += 1;
-            return Some(());
+            Some(())
         }
     }
 
@@ -851,8 +870,8 @@ pub mod morx {
     }
 
     impl<'a> NonContextual<'a> {
-        fn new(data: Bytes<'a>) -> Option<Self> {
-            Some(Self { data })
+        fn new(data: Bytes<'a>) -> Self {
+            Self { data }
         }
 
         /// Returns a substitution for the specified glyph id.
@@ -928,7 +947,15 @@ pub mod morx {
                 inserted = true;
                 f(base, glyphs)?;
             }
-            let advance = entry.flags & DONT_ADVANCE == 0;
+            let mut advance = entry.flags & DONT_ADVANCE == 0;
+            if advance {
+                state.cycles = 0;
+            } else if state.cycles > MAX_CYCLES {
+                state.cycles = 0;
+                advance = true;
+            } else {
+                state.cycles += 1;
+            }
             if inserted {
                 // It doesn't seem likely to find a don't advance flag with an active
                 // insertion state. The only logical place to position the cursor is
@@ -979,6 +1006,7 @@ pub mod morx {
     pub struct InsertionState {
         state: u16,
         mark: usize,
+        cycles: u16,
     }
 
     impl InsertionState {
@@ -1108,7 +1136,7 @@ pub mod kerx {
         let b = if kerx == 0 {
             Bytes::new(&[])
         } else {
-            Bytes::with_offset(data, kerx as usize).unwrap_or(Bytes::new(&[]))
+            Bytes::with_offset(data, kerx as usize).unwrap_or_else(|| Bytes::new(&[]))
         };
         let ankr = if ankr == 0 {
             &[]
@@ -1149,7 +1177,7 @@ pub mod kerx {
             let offset = self.offset;
             let subtable = Subtable::new(&self.data, offset, self.version, self.ankr)?;
             self.offset = self.offset.checked_add(subtable.size as usize)?;
-            return Some(subtable);
+            Some(subtable)
         }
     }
 
@@ -1244,14 +1272,13 @@ pub mod kerx {
             let mut l = 0;
             let mut h = self.count;
             while l < h {
+                use core::cmp::Ordering::*;
                 let i = (l + h) / 2;
                 let pair = b.read::<u32>(base + i * reclen)?;
-                if key > pair {
-                    l = i + 1;
-                } else if key < pair {
-                    h = i;
-                } else {
-                    return b.read_i16(base + i * reclen + 4);
+                match key.cmp(&pair) {
+                    Greater => l = i + 1,
+                    Less => h = i,
+                    Equal => return b.read_i16(base + i * reclen + 4),
                 }
             }
             None
@@ -1327,7 +1354,15 @@ pub mod kerx {
             if entry.flags & RESET != 0 {
                 state.pos = 0;
             }
-            let advance = entry.flags & DONT_ADVANCE == 0;
+            let mut advance = entry.flags & DONT_ADVANCE == 0;
+            if advance {
+                state.cycles = 0;
+            } else if state.cycles > MAX_CYCLES {
+                state.cycles = 0;
+                advance = true;
+            } else {
+                state.cycles += 1;
+            }
             Some(advance as usize)
         }
     }
@@ -1338,6 +1373,7 @@ pub mod kerx {
         state: u16,
         stack: [usize; 8],
         pos: usize,
+        cycles: u16,
     }
 
     impl ContextualState {
@@ -1422,15 +1458,15 @@ pub mod kerx {
                 state.mark_id = glyph_id;
             }
             if entry.data != 0xFFFF {
-                let offset = self
-                    .control_table
-                    .checked_add(entry.data as usize * 2)?;
+                let offset = self.control_table.checked_add(entry.data as usize * 2)?;
                 match self.action_type {
                     0 => {}
                     1 => {
-                        let mark_index =  self.data.read_u16(offset)?;
+                        let mark_index = self.data.read_u16(offset)?;
                         let cur_index = self.data.read_u16(offset + 2)?;
-                        if let Some((x, y)) = self.anchor_offset(mark_index, state.mark_id, cur_index, glyph_id) {
+                        if let Some((x, y)) =
+                            self.anchor_offset(mark_index, state.mark_id, cur_index, glyph_id)
+                        {
                             let diff = index - state.mark;
                             if diff < 255 {
                                 f(index, diff, x, y);
@@ -1441,11 +1477,25 @@ pub mod kerx {
                     _ => {}
                 }
             }
-            let advance = entry.flags & DONT_ADVANCE == 0;
+            let mut advance = entry.flags & DONT_ADVANCE == 0;
+            if advance {
+                state.cycles = 0;
+            } else if state.cycles > MAX_CYCLES {
+                state.cycles = 0;
+                advance = true;
+            } else {
+                state.cycles += 1;
+            }
             Some(advance as usize)
         }
 
-        fn anchor_offset(&self, mark_index: u16, mark_id: u16, cur_index: u16, cur_id: u16) -> Option<(f32, f32)> {
+        fn anchor_offset(
+            &self,
+            mark_index: u16,
+            mark_id: u16,
+            cur_index: u16,
+            cur_id: u16,
+        ) -> Option<(f32, f32)> {
             let mark_point = anchor_points(self.ankr, mark_id)?.get(mark_index as u32)?;
             let cur_point = anchor_points(self.ankr, cur_id)?.get(cur_index as u32)?;
             Some((
@@ -1461,17 +1511,18 @@ pub mod kerx {
         state: u16,
         mark: usize,
         mark_id: u16,
+        cycles: u16,
     }
 
     impl Format4State {
         pub fn new() -> Self {
             Self::default()
         }
-    }    
+    }
 
     /// Returns the set of anchor points for the specified glyph.
     pub fn anchor_points<'a>(data: &'a [u8], glyph_id: u16) -> Option<AnchorPoints<'a>> {
-        if data.len() == 0 {
+        if data.is_empty() {
             return None;
         }
         let b = Bytes::new(data);
@@ -1517,7 +1568,7 @@ pub mod kern {
         let b = if kern == 0 {
             Bytes::new(&[])
         } else {
-            Bytes::with_offset(data, kern as usize).unwrap_or(Bytes::new(&[]))
+            Bytes::with_offset(data, kern as usize).unwrap_or_else(|| Bytes::new(&[]))
         };
         let version = b.read_or_default::<u16>(0);
         if version == 0 {
@@ -1537,7 +1588,7 @@ pub mod kern {
                 len,
                 cur: 0,
                 is_aat: true,
-            }            
+            }
         }
     }
 
@@ -1546,13 +1597,13 @@ pub mod kern {
         data: Bytes<'a>,
         offset: usize,
         len: u32,
-        cur: u32,        
+        cur: u32,
         is_aat: bool,
     }
 
     impl<'a> Iterator for Subtables<'a> {
         type Item = Subtable<'a>;
-        
+
         fn next(&mut self) -> Option<Self::Item> {
             if self.cur >= self.len {
                 return None;
@@ -1561,7 +1612,7 @@ pub mod kern {
             let offset = self.offset;
             let subtable = Subtable::new(&self.data, offset, self.is_aat)?;
             self.offset = self.offset.checked_add(subtable.size as usize)?;
-            return Some(subtable);
+            Some(subtable)
         }
     }
 
@@ -1576,7 +1627,7 @@ pub mod kern {
         is_horizontal: bool,
         cross_stream: bool,
         format: u8,
-    }    
+    }
 
     impl<'a> Subtable<'a> {
         fn new(data: &Bytes<'a>, mut offset: usize, is_aat: bool) -> Option<Self> {
@@ -1611,8 +1662,8 @@ pub mod kern {
                 cross_stream,
                 format,
             })
-        }    
-        
+        }
+
         pub fn is_horizontal(&self) -> bool {
             self.is_horizontal
         }
@@ -1644,7 +1695,7 @@ pub mod kern {
     }
 
     impl<'a> Format0<'a> {
-        fn new(subtable: &Subtable<'a>) -> Option<Self> {            
+        fn new(subtable: &Subtable<'a>) -> Option<Self> {
             let count = subtable.data.read_u16(subtable.offset)? as usize;
             Some(Self {
                 data: subtable.data,
@@ -1662,19 +1713,18 @@ pub mod kern {
             let mut l = 0;
             let mut h = self.count;
             while l < h {
+                use core::cmp::Ordering::*;
                 let i = (l + h) / 2;
                 let pair = b.read::<u32>(base + i * reclen)?;
-                if key > pair {
-                    l = i + 1;
-                } else if key < pair {
-                    h = i;
-                } else {
-                    return b.read_i16(base + i * reclen + 4);
+                match key.cmp(&pair) {
+                    Greater => l = i + 1,
+                    Less => h = i,
+                    Equal => return b.read_i16(base + i * reclen + 4),
                 }
             }
             None
         }
-    }    
+    }
 
     /// Contextual kerning subtable.
     #[derive(Copy, Clone)]
@@ -1729,7 +1779,7 @@ pub mod kern {
                     let pos = state.pos - 1;
                     state.pos = pos;
                     if self.cross_stream && value as u16 == 0x8000 {
-                        // Reset cross stream?                        
+                        // Reset cross stream?
                     } else {
                         f(state.stack[pos], value)?;
                     }
@@ -1737,10 +1787,18 @@ pub mod kern {
                         state.pos = 0;
                         break;
                     }
-                    value_offset = value_offset + 2;
+                    value_offset += 2;
                 }
             }
-            let advance = entry.flags & DONT_ADVANCE == 0;
+            let mut advance = entry.flags & DONT_ADVANCE == 0;
+            if advance {
+                state.cycles = 0;
+            } else if state.cycles > MAX_CYCLES {
+                state.cycles = 0;
+                advance = true;
+            } else {
+                state.cycles += 1;
+            }
             Some(advance as usize)
         }
     }
@@ -1751,6 +1809,7 @@ pub mod kern {
         state: u16,
         stack: [usize; 8],
         pos: usize,
+        cycles: u16,
     }
 
     impl Format1State {
