@@ -1,6 +1,6 @@
 //! OpenType advanced typography tables.
 
-use super::{raw_tag, RawTag, Bytes};
+use super::{raw_tag, Bytes, RawTag};
 
 pub const GDEF: RawTag = raw_tag(b"GDEF");
 pub const GSUB: RawTag = raw_tag(b"GSUB");
@@ -118,15 +118,9 @@ impl<'a> Gdef<'a> {
 
     pub fn delta(&self, outer: u16, inner: u16, coords: &[i16]) -> f32 {
         if self.var_store != 0 {
-            super::var::item_delta(
-                self.data.data(),
-                self.var_store,
-                outer,
-                inner,
-                coords,
-            )
-            .map(|d| d.to_f32())
-            .unwrap_or(0.)
+            super::var::item_delta(self.data.data(), self.var_store, outer, inner, coords)
+                .map(|d| d.to_f32())
+                .unwrap_or(0.)
         } else {
             0.
         }
@@ -158,7 +152,7 @@ pub struct LookupData {
     pub index: u16,
     pub stage: u8,
     pub kind: LookupKind,
-    pub feature: u8,
+    pub feature: u16,
     pub mask: u8,
     pub ignored: u8,
     pub is_ext: bool,
@@ -230,7 +224,7 @@ pub struct FeatureSubsts(u32);
 
 impl FeatureSubsts {
     pub fn new(b: &Bytes, offset: u32, coords: &[i16]) -> Option<Self> {
-        if offset == 0 || coords.len() == 0 {
+        if offset == 0 || coords.is_empty() {
             return None;
         }
         let base = offset as usize;
@@ -275,15 +269,14 @@ impl FeatureSubsts {
         let mut l = 0;
         let mut h = count;
         while l < h {
+            use core::cmp::Ordering::*;
             let i = (l + h) / 2;
             let rec = base + i * 6;
             let idx = b.read::<u16>(rec)?;
-            if index < idx {
-                h = i;
-            } else if index > idx {
-                l = i + 1;
-            } else {
-                return Some((self.0 + b.read::<u32>(rec + 2)?) as usize);
+            match index.cmp(&idx) {
+                Less => h = i,
+                Greater => l = i + 1,
+                Equal => return Some((self.0 + b.read::<u32>(rec + 2)?) as usize),
             }
         }
         None
@@ -323,15 +316,14 @@ pub fn script_by_tag(b: &Bytes, gsubgpos_offset: u32, script: RawTag) -> Option<
     let mut l = 0;
     let mut h = b.read::<u16>(sbase)? as usize;
     while l < h {
+        use core::cmp::Ordering::*;
         let i = l + (h - l) / 2;
         let rec = sbase + 2 + i * 6;
         let t = b.read::<u32>(rec)?;
-        if script < t {
-            h = i;
-        } else if script > t {
-            l = i + 1;
-        } else {
-            return Some(sbase as u32 + b.read::<u16>(rec + 4)? as u32);
+        match script.cmp(&t) {
+            Less => h = i,
+            Greater => l = i + 1,
+            Equal => return Some(sbase as u32 + b.read::<u16>(rec + 4)? as u32),
         }
     }
     None
@@ -363,10 +355,7 @@ pub fn script_language_at(b: &Bytes, script_offset: u32, index: u16) -> Option<(
         return None;
     }
     let index = if index == 0 {
-        return Some((
-            DFLT,
-            script_default_language(b, script_offset)?,
-        ));
+        return Some((DFLT, script_default_language(b, script_offset)?));
     } else {
         index - 1
     };
@@ -392,19 +381,20 @@ pub fn script_language_by_tag(
         let mut l = 0;
         let mut h = b.read::<u16>(base + 2)? as usize;
         while l < h {
+            use core::cmp::Ordering::*;
             let i = (l + h) / 2;
             let rec = base + 4 + i * 6;
             let t = b.read::<u32>(rec)?;
-            if lang < t {
-                h = i;
-            } else if lang > t {
-                l = i + 1;
-            } else {
-                let lang_offset = b.read::<u16>(rec + 4)? as usize;
-                if lang_offset == 0 {
-                    return None;
+            match lang.cmp(&t) {
+                Less => h = i,
+                Greater => l = i + 1,
+                Equal => {
+                    let lang_offset = b.read::<u16>(rec + 4)? as usize;
+                    if lang_offset == 0 {
+                        return None;
+                    }
+                    return Some((script_offset + lang_offset as u32, false));
                 }
-                return Some((script_offset + lang_offset as u32, false));
             }
         }
     }
@@ -412,7 +402,7 @@ pub fn script_language_by_tag(
     if default == 0 {
         return None;
     }
-    return Some(((base + default) as u32, true));
+    Some(((base + default) as u32, true))
 }
 
 pub fn language_or_default_by_tags(
@@ -649,8 +639,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -661,8 +651,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -673,8 +663,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -685,8 +675,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -698,8 +688,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -711,8 +701,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -723,8 +713,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -735,8 +725,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -747,8 +737,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -759,8 +749,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
             };
             let coverage = cov(b, base, 2)?;
             Some(SubtableData {
-                kind,
                 offset,
+                kind,
                 coverage,
             })
         }
@@ -773,8 +763,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
                 };
                 let coverage = cov(b, base, 2)?;
                 Some(SubtableData {
-                    kind,
                     offset,
+                    kind,
                     coverage,
                 })
             }
@@ -786,7 +776,7 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
                     coverage,
                 })
             }
-            _ => return None,
+            _ => None,
         },
         ChainContext => match fmt {
             1 | 2 => {
@@ -797,8 +787,8 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
                 };
                 let coverage = cov(b, base, 2)?;
                 Some(SubtableData {
-                    kind,
                     offset,
+                    kind,
                     coverage,
                 })
             }
@@ -815,9 +805,20 @@ pub fn subtable_data(b: &Bytes, offset: u32, kind: LookupKind, fmt: u16) -> Opti
                     coverage,
                 })
             }
-            _ => return None,
+            _ => None,
         },
-        _ => None,
+        RevChainContext => {
+            let kind = match fmt {
+                1 => SubtableKind::RevChainContext1,
+                _ => return None,
+            };
+            let coverage = cov(b, base, 2)?;
+            Some(SubtableData {
+                offset,
+                kind,
+                coverage,
+            })
+        }
     }
 }
 
@@ -857,14 +858,13 @@ pub unsafe fn fast_coverage(b: &Bytes, coverage_offset: u32, glyph_id: u16) -> O
         let mut l = 0;
         let mut h = len;
         while l < h {
+            use core::cmp::Ordering::*;
             let i = (l + h) / 2;
             let g = b.read_unchecked::<u16>(arr + i * 2);
-            if glyph_id < g {
-                h = i;
-            } else if glyph_id > g {
-                l = i + 1;
-            } else {
-                return Some(i as u16);
+            match glyph_id.cmp(&g) {
+                Less => h = i,
+                Greater => l = i + 1,
+                Equal => return Some(i as u16),
             }
         }
     } else if fmt == 2 {
@@ -902,14 +902,13 @@ pub fn coverage(b: &Bytes, coverage_offset: u32, glyph_id: u16) -> Option<u16> {
         let mut l = 0;
         let mut h = len;
         while l < h {
+            use core::cmp::Ordering::*;
             let i = (l + h) / 2;
             let g = unsafe { b.read_unchecked::<u16>(arr + i * 2) };
-            if glyph_id < g {
-                h = i;
-            } else if glyph_id > g {
-                l = i + 1;
-            } else {
-                return Some(i as u16);
+            match glyph_id.cmp(&g) {
+                Less => h = i,
+                Greater => l = i + 1,
+                Equal => return Some(i as u16),
             }
         }
     } else if fmt == 2 {
