@@ -1,7 +1,7 @@
-use super::cluster::Glyph;
+use super::cluster::{Glyph, GlyphInfo};
 use super::feature::*;
 use crate::text::{
-    cluster::{CharCluster, ClusterInfo, Char, ShapeClass, SourceRange, MAX_CLUSTER_SIZE},
+    cluster::{Char, CharCluster, ClusterInfo, ShapeClass, SourceRange, MAX_CLUSTER_SIZE},
     JoiningType,
 };
 use core::ops::Range;
@@ -16,7 +16,7 @@ pub const IGNORABLE: u16 = 64;
 
 /// Per glyph shaping data.
 #[derive(Copy, Clone, Default, Debug)]
-pub struct GlyphInfo {
+pub struct GlyphData {
     pub id: u16,
     pub flags: u16,
     pub class: u8,
@@ -30,7 +30,7 @@ pub struct GlyphInfo {
     pub data: u32,
 }
 
-impl GlyphInfo {
+impl GlyphData {
     pub fn is_component(&self) -> bool {
         self.flags & COMPONENT != 0
     }
@@ -38,7 +38,7 @@ impl GlyphInfo {
 
 /// Per glyph shaping position data.
 #[derive(Copy, Clone, Default, Debug)]
-pub struct PositionInfo {
+pub struct PositionData {
     pub base: u8,
     pub flags: u16,
     pub x: f32,
@@ -47,9 +47,10 @@ pub struct PositionInfo {
 }
 
 impl Glyph {
-    pub(super) fn new(g: &GlyphInfo, p: &PositionInfo) -> Self {
+    pub(super) fn new(g: &GlyphData, p: &PositionData) -> Self {
         Self {
             id: g.id,
+            info: GlyphInfo(p.flags),
             x: p.x,
             y: p.y,
             advance: p.advance,
@@ -60,8 +61,8 @@ impl Glyph {
 
 #[derive(Clone, Default)]
 pub struct Buffer {
-    pub glyphs: Vec<GlyphInfo>,
-    pub positions: Vec<PositionInfo>,
+    pub glyphs: Vec<GlyphData>,
+    pub positions: Vec<PositionData>,
     pub infos: Vec<(ClusterInfo, bool, u32)>,
     pub ranges: Vec<SourceRange>,
     pub shaped_glyphs: Vec<Glyph>,
@@ -177,7 +178,7 @@ impl Buffer {
     #[inline(always)]
     fn push_char(&mut self, ch: &Char) {
         let cluster = self.next_cluster;
-        self.glyphs.push(GlyphInfo {
+        self.glyphs.push(GlyphData {
             id: ch.glyph_id,
             flags: (ch.ignorable as u16) << 6,
             class: 0,
@@ -195,16 +196,16 @@ impl Buffer {
     fn _push_hangul_char(&mut self, ch: &Char) {
         let cluster = self.next_cluster;
         let c = ch.ch as u32;
-        let mask = if (c >= 0x1100 && c <= 0x115F) || (c >= 0xA960 && c <= 0xA97C) {
+        let mask = if (0x1100..=0x115F).contains(&c) || (0xA960..=0xA97C).contains(&c) {
             1
-        } else if (c >= 0x1160 && c <= 0x11A7) || (c >= 0xD7B0 && c <= 0xD7C6) {
+        } else if (0x1160..=0x11A7).contains(&c) || (0xD7B0..=0xD7C6).contains(&c) {
             2
-        } else if (c >= 0x11A8 && c <= 0x11FF) || (c >= 0xD7CB && c <= 0xD7FB) {
+        } else if (0x11A8..=0x11FF).contains(&c) || (0xD7CB..=0xD7FB).contains(&c) {
             4
         } else {
             1 | 2 | 4
         };
-        self.glyphs.push(GlyphInfo {
+        self.glyphs.push(GlyphData {
             id: ch.glyph_id,
             flags: (ch.ignorable as u16) << 6,
             class: 0,
@@ -220,7 +221,8 @@ impl Buffer {
     }
 
     fn push_cluster(&mut self, cluster: &CharCluster) {
-        self.infos.push((cluster.info(), false, cluster.user_data()));
+        self.infos
+            .push((cluster.info(), false, cluster.user_data()));
         self.ranges.push(cluster.range());
     }
 
@@ -265,12 +267,11 @@ impl Buffer {
             self.glyphs
                 .retain(|g| g.flags & COMPONENT == 0 && g.id != 0xFFFF);
         } else {
-            self.glyphs
-                .retain(|g| g.flags & COMPONENT == 0);
+            self.glyphs.retain(|g| g.flags & COMPONENT == 0);
         }
         self.positions.clear();
         self.positions
-            .resize(self.glyphs.len(), PositionInfo::default());
+            .resize(self.glyphs.len(), PositionData::default());
     }
 
     pub fn substitute(&mut self, index: usize, id: u16) {
@@ -290,7 +291,7 @@ impl Buffer {
         //     }
         //     println!(" -> {}", id);
         // }
-        if components.len() == 0 {
+        if components.is_empty() {
             return;
         }
         let g = &mut self.glyphs[index];
@@ -348,7 +349,11 @@ impl Buffer {
     }
 
     pub fn multiply(&mut self, index: usize, count: usize) {
-        let g = self.glyphs[index];
+        let g = self
+            .glyphs
+            .get(index)
+            .copied()
+            .unwrap_or_else(GlyphData::default);
         self.glyphs.splice(index..index, (0..count).map(|_| g));
     }
 
@@ -409,12 +414,12 @@ impl Buffer {
 
 struct SubstIter<'a> {
     ids: &'a [u16],
-    g: GlyphInfo,
+    g: GlyphData,
     cur: usize,
 }
 
 impl<'a> Iterator for SubstIter<'a> {
-    type Item = GlyphInfo;
+    type Item = GlyphData;
 
     fn size_hint(&self) -> (usize, Option<usize>) {
         let remaining = self.ids.len() - self.cur;
@@ -425,7 +430,7 @@ impl<'a> Iterator for SubstIter<'a> {
         if self.cur >= self.ids.len() {
             return None;
         }
-        let g = GlyphInfo {
+        let g = GlyphData {
             id: self.ids[self.cur],
             flags: SUBSTITUTED,
             ..self.g
@@ -533,7 +538,8 @@ pub fn reorder_myanmar(chars: &[Char], order: &mut Vec<usize>) {
     }
 }
 
-pub fn reorder_complex(glyphs: &mut [GlyphInfo], buf: &mut Vec<GlyphInfo>, order: &mut Vec<usize>) {
+#[allow(clippy::needless_range_loop)]
+pub fn reorder_complex(glyphs: &mut [GlyphData], buf: &mut Vec<GlyphData>, order: &mut Vec<usize>) {
     use ShapeClass::*;
     let mut first_base = None;
     let mut last_base = None;
@@ -545,7 +551,7 @@ pub fn reorder_complex(glyphs: &mut [GlyphInfo], buf: &mut Vec<GlyphInfo>, order
     let mut ignored = [false; 64];
     let len = glyphs.len();
     if buf.len() < glyphs.len() {
-        buf.resize(len, GlyphInfo::default());
+        buf.resize(len, GlyphData::default());
     }
     let buf = &mut buf[..len];
     if order.len() < len {
@@ -613,19 +619,19 @@ pub fn reorder_complex(glyphs: &mut [GlyphInfo], buf: &mut Vec<GlyphInfo>, order
                 j += 1;
             }
         }
-        if let Some(i) = pref.clone() {
+        if let Some(i) = pref {
             order[j] = i;
             j += 1;
         }
     }
     // Insert the base...
-    if let Some(i) = first_base.clone() {
+    if let Some(i) = first_base {
         order[j] = i;
         j += 1;
     }
     if last_base.is_none() {
         // ... and the reph
-        if let Some(i) = reph.clone() {
+        if let Some(i) = reph {
             order[j] = i;
             j += 1;
         }
@@ -643,7 +649,7 @@ pub fn reorder_complex(glyphs: &mut [GlyphInfo], buf: &mut Vec<GlyphInfo>, order
         j += 1;
         // Insert reph after final base
         if Some(i) == last_base {
-            if let Some(i) = reph.clone() {
+            if let Some(i) = reph {
                 order[j] = i;
                 j += 1;
             }
@@ -662,7 +668,7 @@ pub fn reorder_complex(glyphs: &mut [GlyphInfo], buf: &mut Vec<GlyphInfo>, order
                     j += 1;
                 }
             }
-            if let Some(i) = pref.clone() {
+            if let Some(i) = pref {
                 order[j] = i;
                 j += 1;
             }
@@ -684,8 +690,8 @@ pub fn reorder_complex(glyphs: &mut [GlyphInfo], buf: &mut Vec<GlyphInfo>, order
 // Matches the Arabic joining table from Harfbuzz.
 #[rustfmt::skip]
 const JOIN_STATES: [[(u8, u8, u8); 6]; 7] = [
-    //   U,                            L,                       R,                        D,                    ALAPH,                 DALATH_RISH      
-    // State 0: prev was U, not willing to join. 
+    //   U,                            L,                       R,                        D,                    ALAPH,                 DALATH_RISH
+    // State 0: prev was U, not willing to join.
     [ (NONE_MASK,NONE_MASK,0), (NONE_MASK,ISOL_MASK,2), (NONE_MASK,ISOL_MASK,1), (NONE_MASK,ISOL_MASK,2), (NONE_MASK,ISOL_MASK,1), (NONE_MASK,ISOL_MASK,6), ],
     // State 1: prev was R or ISOL_MASK/ALAPH, not willing to join.
     [ (NONE_MASK,NONE_MASK,0), (NONE_MASK,ISOL_MASK,2), (NONE_MASK,ISOL_MASK,1), (NONE_MASK,ISOL_MASK,2), (NONE_MASK,FIN2_MASK,5), (NONE_MASK,ISOL_MASK,6), ],
