@@ -1,8 +1,8 @@
 /*!
 Mapping complex text to a sequence of positioned glyphs.
 
-Shaping is the process of converting a sequence of 
-[character clusters](CharCluster) into a sequence of 
+Shaping is the process of converting a sequence of
+[character clusters](CharCluster) into a sequence of
 [glyph clusters](GlyphCluster) with respect to the rules of a particular
 writing system and the typographic features available in a font. The shaper
 operates on one _item_ at a time where an item is a run of text with
@@ -20,7 +20,7 @@ this amortizes the cost of allocations, reduces contention for the global
 heap and increases the hit rate for the internal acceleration structures. If
 you're doing multithreaded layout, you should keep a context per thread.
 
-The only method available on the context is [`builder`](ShapeContext::builder) 
+The only method available on the context is [`builder`](ShapeContext::builder)
 which takes a type that can be converted into a [`FontRef`] as an argument
 and produces a [`ShaperBuilder`] that provides options for configuring and
 building a [`Shaper`].
@@ -39,8 +39,8 @@ let mut shaper = context.builder(font)
 ```
 
 You can specify feature settings by calling the [`features`](ShaperBuilder::features)
-method with an iterator that yields a sequence of values that are convertible 
-to [`Setting<u16>`]. Tuples of (&str, u16) will work in a pinch. For example, 
+method with an iterator that yields a sequence of values that are convertible
+to [`Setting<u16>`]. Tuples of (&str, u16) will work in a pinch. For example,
 you can enable discretionary ligatures like this:
 ```
 # use swash::{FontRef, CacheKey, shape::*, text::Script, tag_from_bytes};
@@ -109,7 +109,7 @@ line break opportunities through the shaper.
 This also provides a junction point for inserting a font fallback
 mechanism.
 
-All of this is served by the functionality in the 
+All of this is served by the functionality in the
 [`text::cluster`](crate::text::cluster) module.
 
 Let's see a somewhat contrived example that demonstrates the process:
@@ -157,11 +157,11 @@ Phew! That's quite a lot of work. It also happens to be exactly what
 
 So why bother? As mentioned earlier, this method allows you to customize
 the per-character data that passes through the shaper. Is your source text in
-UTF-16 instead of UTF-8? No problem. Set the [`offset`](Token::offset) and 
+UTF-16 instead of UTF-8? No problem. Set the [`offset`](Token::offset) and
 [`len`](Token::len) fields of your [`Token`]s to appropriate values. Are you shaping
-across style spans? Set the [`data`](Token::data) field to the index of your span so 
+across style spans? Set the [`data`](Token::data) field to the index of your span so
 it can be recovered. Have you used the
-[`Analyze`](crate::text::Analyze) iterator to generate 
+[`Analyze`](crate::text::Analyze) iterator to generate
 [`CharInfo`](crate::text::cluster::CharInfo)s containing boundary analysis? This
 is where you apply them to the [`info`](Token::info) fields of your [`Token`]s.
 
@@ -221,7 +221,7 @@ candidate font.
 Since this process is done during shaping, upon return we compare the selected
 font with our current font and if they're different, we complete shaping for the
 clusters submitted so far and continue the process by building a new shaper with
-the selected font. By doing manual cluster parsing and nominal glyph mapping 
+the selected font. By doing manual cluster parsing and nominal glyph mapping
 _outside_ the shaper, we can implement per-cluster font fallback without the costly
 technique of heuristically shaping runs.
 
@@ -234,7 +234,7 @@ method in which you store it will depend entirely on the design of your text lay
 system.
 
 Please note that, unlike HarfBuzz, this shaper does _not_ reverse runs that are in
-right-to-left order. The reasoning is that, for correctness, line breaking must be 
+right-to-left order. The reasoning is that, for correctness, line breaking must be
 done in logical order and reversing runs should occur during bidi reordering.
 
 Also pertinent to right-to-left runs: you'll need to ensure that you reverse
@@ -244,6 +244,9 @@ for proper mark placement.
 
 pub mod cluster;
 
+#[doc(hidden)]
+pub mod partition;
+
 mod aat;
 mod at;
 mod buffer;
@@ -261,7 +264,7 @@ use crate::text::{
     cluster::{CharCluster, Parser, ShapeClass, Token},
     Language, Script,
 };
-use at::{FeatureStore, FeatureStoreBuilder};
+use at::{FeatureMask, FeatureStore, FeatureStoreBuilder};
 use buffer::*;
 use cache::{FeatureCache, FontEntry};
 use core::borrow::Borrow;
@@ -277,7 +280,7 @@ pub enum Direction {
 }
 
 /// Context that manages caches and transient buffers for shaping.
-/// 
+///
 /// See the module level [documentation](index.html#building-the-shaper) for detail.
 pub struct ShapeContext {
     font_cache: FontCache<FontEntry>,
@@ -311,11 +314,17 @@ impl ShapeContext {
     }
 }
 
+impl Default for ShapeContext {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 struct State {
     buffer: Buffer,
     store_builder: FeatureStoreBuilder,
     order: Vec<usize>,
-    glyphs: Vec<GlyphInfo>,
+    glyphs: Vec<GlyphData>,
     disable_kern: bool,
     features: Vec<(u32, u16)>,
     selectors: Vec<(u16, u16)>,
@@ -342,8 +351,8 @@ impl State {
 }
 
 /// Builder for configuring a shaper.
-/// 
-/// See the module level [documentation](index.html#building-the-shaper) for more detail. 
+///
+/// See the module level [documentation](index.html#building-the-shaper) for more detail.
 pub struct ShaperBuilder<'a> {
     state: &'a mut State,
     feature_cache: &'a mut FeatureCache,
@@ -353,6 +362,7 @@ pub struct ShaperBuilder<'a> {
     coords: &'a mut Vec<i16>,
     charmap: Charmap<'a>,
     dotted_circle: Option<u16>,
+    retain_ignorables: bool,
     size: f32,
     script: Script,
     lang: Option<Language>,
@@ -376,6 +386,7 @@ impl<'a> ShaperBuilder<'a> {
             coords: &mut context.coords,
             charmap: font_entry.charmap.materialize(&font),
             dotted_circle: None,
+            retain_ignorables: false,
             size: 0.,
             script: Script::Latin,
             lang: None,
@@ -435,10 +446,12 @@ impl<'a> ShaperBuilder<'a> {
             self.coords.resize(vars.len(), 0);
             for setting in settings {
                 let setting = setting.into();
-                for var in vars.clone() {
+                for var in vars {
                     if var.tag() == setting.tag {
                         let value = var.normalize(setting.value);
-                        self.coords.get_mut(var.index()).map(|c| *c = value);
+                        if let Some(c) = self.coords.get_mut(var.index()) {
+                            *c = value;
+                        }
                     }
                 }
             }
@@ -471,6 +484,13 @@ impl<'a> ShaperBuilder<'a> {
         self
     }
 
+    /// Specifies whether characters defined as default ignorable should be
+    /// retained by the shaper. The default is `false`.
+    pub fn retain_ignorables(mut self, yes: bool) -> Self {
+        self.retain_ignorables = yes;
+        self
+    }
+
     /// Builds a shaper for the current configuration.
     pub fn build(self) -> Shaper<'a> {
         let engine = Engine::new(
@@ -506,13 +526,14 @@ impl<'a> ShaperBuilder<'a> {
             );
             (Some(store as _), sub, pos)
         } else {
-            (None, 0, 0)
+            (None, FeatureMask::default(), FeatureMask::default())
         };
         Shaper {
             state: self.state,
             font: self.font,
             font_entry: self.font_entry,
             charmap: self.charmap,
+            retain_ignorables: self.retain_ignorables,
             size: self.size,
             script: self.script,
             joined: engine.use_ot && self.script.is_joined(),
@@ -527,21 +548,22 @@ impl<'a> ShaperBuilder<'a> {
 
 /// Maps character clusters to positioned glyph clusters according to
 /// typographic rules and features.
-/// 
+///
 /// See the module level [documentation](index.html#feeding-the-shaper) for detail.
 pub struct Shaper<'a> {
     state: &'a mut State,
     font: FontRef<'a>,
     font_entry: &'a FontEntry,
     charmap: Charmap<'a>,
+    retain_ignorables: bool,
     size: f32,
     script: Script,
     joined: bool,
     dir: Direction,
     engine: Engine<'a>,
     store: Option<&'a FeatureStore>,
-    sub_mask: u64,
-    pos_mask: u64,
+    sub_mask: FeatureMask,
+    pos_mask: FeatureMask,
 }
 
 impl<'a> Shaper<'a> {
@@ -560,7 +582,7 @@ impl<'a> Shaper<'a> {
                 let range = buf.push_order(cluster, &self.state.order);
                 e.set_classes(buf, Some(range.clone()));
                 let start = range.start;
-                e.gsub(s, s.groups.default, buf, Some(range.clone()));
+                e.gsub(s, s.groups.default, buf, Some(range));
                 let end = buf.len();
                 e.gsub(s, s.groups.reph, buf, Some(start..end));
                 let end = buf.len();
@@ -578,6 +600,12 @@ impl<'a> Shaper<'a> {
                 let start = range.start;
                 // Default group
                 e.gsub(s, s.groups.default, buf, Some(range.clone()));
+                for g in &mut buf.glyphs[range] {
+                    if g.char_class == ShapeClass::Halant && g.flags & SUBSTITUTED != 0 {
+                        // Don't prevent reordering across a virama that has been substituted
+                        g.char_class = ShapeClass::Other;
+                    }
+                }
                 // Reph identification
                 let len = 3.min(buf.glyphs.len() - start);
                 let end = start + len;
@@ -656,7 +684,10 @@ impl<'a> Shaper<'a> {
         self.finish();
         let buf = &mut self.state.buffer;
         buf.shaped_glyphs.clear();
-        let mut sentinel = (buffer::GlyphInfo::default(), buffer::PositionInfo::default());
+        let mut sentinel = (
+            buffer::GlyphData::default(),
+            buffer::PositionData::default(),
+        );
         sentinel.0.cluster = buf.ranges.len() as u32;
         let mut last_cluster = 0;
         for (g, p) in buf
@@ -666,9 +697,8 @@ impl<'a> Shaper<'a> {
             .chain(core::iter::once((&sentinel.0, &sentinel.1)))
         {
             if g.cluster != last_cluster {
-                let diff = g.cluster - last_cluster;
                 // Simple and common case: no ligatures and no empty clusters.
-                if diff == 1 {
+                if last_cluster > g.cluster || g.cluster - last_cluster == 1 {
                     let index = last_cluster as usize;
                     let info = &buf.infos[index];
                     let cluster = GlyphCluster {
@@ -729,7 +759,7 @@ impl<'a> Shaper<'a> {
                 }
             }
             last_cluster = g.cluster;
-            if g.flags & IGNORABLE == 0 {
+            if self.retain_ignorables || g.flags & IGNORABLE == 0 {
                 buf.shaped_glyphs.push(Glyph::new(g, p));
             }
         }
@@ -753,7 +783,7 @@ impl<'a> Shaper<'a> {
 
     fn finish(&mut self) {
         use engine::{PosMode, SubMode};
-        if self.state.buffer.glyphs.len() == 0 {
+        if self.state.buffer.glyphs.is_empty() {
             return;
         }
         let e = &mut self.engine;
@@ -813,7 +843,7 @@ impl<'a> Shaper<'a> {
             .metrics
             .materialize_glyph_metrics(&self.font, self.engine.coords);
         for (g, p) in buf.glyphs.iter_mut().zip(buf.positions.iter_mut()) {
-            if g.class != 3 {
+            if g.flags & MARK_ATTACH == 0 {
                 p.advance += glyph_metrics.advance_width(g.id);
             }
             g.flags |= p.flags;
