@@ -2,6 +2,9 @@
 Character properties and textual analysis.
 */
 
+// Avoid errors for generated Unicode data.
+#![allow(clippy::upper_case_acronyms)]
+
 mod analyze;
 mod compose;
 mod lang;
@@ -11,6 +14,6 @@ mod unicode_data;
 
 pub mod cluster;
 
+pub use analyze::{analyze, Analyze};
 pub use lang::{Cjk, Language};
 pub use unicode::*;
-pub use analyze::{analyze, Analyze};
